from .base import PrimitiveType

__all__ = [
    "UserNameType",
]


class UserNameType(PrimitiveType):
    def __init__(
        self,
        key,
        value,
        name="username-type",
    ):

        super().__init__(
            key=key,
            value=value,
            name=name,
        )

    def __eq__(self, __o):
        return isinstance(__o, type(self)) and (self.value == __o.value)

    def __str__(self) -> str:
        return f"{self.key}"



from walle.core import PrimitiveType

import pytest


class DummyType(PrimitiveType):
    def __init__(
        self,
        key,
        value,
        name,
    ):
        super().__init__(
            key,
            value,
            name,
        )

    def __eq__(self, __o: object) -> bool:
        return isinstance(__o, type(self)) and (__o.key == self.key)

    def __str__(self):
        return f"{self.name}"


@pytest.mark.parametrize(
    "test_input,expected",
    [
        (
            {
                "key": "username",
                "value": "rube",
                "name": "dummy-type",
            },
            {
                "key": "username",
                "value": "rube",
                "name": "dummy-type",
            },
        ),
    ],
)
def test_eval(test_input, expected):
    got = DummyType(**test_input)
    assert got.name == expected["name"]
    assert got.key == expected["key"]
    assert got.value == expected["value"]
    expected = DummyType(**expected)
    assert got == expected
    assert str(got) == expected.name



from abc import ABCMeta, abstractmethod
from walle.core import constants

__all__ = [
    "PrimitiveType",
]


class PrimitiveType(metaclass=ABCMeta):
    """Primitive Base Class"""

    def __init__(
        self,
        key,
        value,
        name="base-type",
        mutable=True,
    ):
        if "base" in name:
            raise AssertionError(f"Expected a specific Type name but got {name}")

        try:
            self.key = getattr(constants, key.upper())
            print(f"Found {key}! Type Created.")
        except:
            raise FileNotFoundError("Expected to find {key}, but it does not exist!")

        self.value = value
        self.name = name
        self.mutable = mutable

    def get_value(self):
        """getter for specific type value"""
        return self.value

    def set_value(self, value):
        """setter method for this types value"""
        self.value = value

    def is_mutable(self):
        return self.mutable

    @abstractmethod
    def __eq__(self, __o: object) -> bool:
        pass

    @abstractmethod
    def __str__(self) -> str:
        pass
